
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Catmandu modules for working with MARC data",
  "dist_author" => [
    "Patrick Hochstenbach, C<< <patrick.hochstenbach at ugent.be> >>"
  ],
  "dist_name" => "Catmandu-MARC",
  "dist_version" => "1.17",
  "license" => "perl",
  "module_name" => "Catmandu::MARC",
  "recursive_test_files" => 1,
  "requires" => {
    "Carp" => 0,
    "Catmandu" => "1.0602",
    "JSON::XS" => "2.3",
    "List::Util" => 0,
    "MARC::File::MARCMaker" => "0.05",
    "MARC::File::MiJ" => "0.04",
    "MARC::File::XML" => "v1.0.5",
    "MARC::Lint" => 0,
    "MARC::Parser::RAW" => 0,
    "MARC::Record" => "v2.0.6",
    "MARC::Spec" => "v1.0.0",
    "Memoize" => 0,
    "Moo" => "1.0",
    "MooX::Singleton" => 0,
    "YAML::XS" => "0.34",
    "perl" => "v5.10.1"
  },
  "test_requires" => {
    "Pod::Simple::HTML" => "3.23",
    "Test::Deep" => 0,
    "Test::More" => "1.001003",
    "Test::Pod" => 0,
    "Test::Simple" => "1.001003",
    "Test::Warn" => 0,
    "Test::Warnings" => 0,
    "XML::XPath" => "1.13"
  }
);


my %fallback_build_requires = (
  "Module::Build" => "0.28",
  "Pod::Simple::HTML" => "3.23",
  "Test::Deep" => 0,
  "Test::More" => "1.001003",
  "Test::Pod" => 0,
  "Test::Simple" => "1.001003",
  "Test::Warn" => 0,
  "Test::Warnings" => 0,
  "XML::XPath" => "1.13"
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
